#! /usr/bin/perl
#
# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006-2007 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


use strict;
use warnings FATAL => 'all';
use diagnostics;

use Wanha::HTTP;
use Wanha::HTML;
use Wanha::Session;
use Wanha::DB;
use Wanha::Link;
use Wanha::Lang;
use Wanha::Config;


=item error
Print error

0: error
=cut
sub error($) {
	my ($error) = @_;
	&Wanha::HTTP::print_header();
	&Wanha::HTML::print_header();
	&Wanha::Lang::printf("<h1><&txt-title-error&></h1>\n");
	&Wanha::Lang::printf("<p><&$error&></p>\n");
	&Wanha::HTML::print_footer();

}


=item try_set_session
Try to set session

0: sid
1: phase
2: go
=cut
sub try_set_session($$$) {
	my ($sid, $phase, $go) = @_;

	if ($phase == 1) {
		if (! defined($sid)) {
			$sid = &Wanha::Session::create_sid();
			if (! defined($sid)) {
				&error("cant-create-session");
				exit(0);
			}
		}
		&Wanha::HTTP::set_cookie("sid", $sid,
				$Wanha::Config::cookie_age);
		&Wanha::HTTP::redirect(&Wanha::Link::session(2, $go));
		&Wanha::HTTP::print_header();
		exit(0);
	}

	if ($phase == 2) {
		if (! defined($sid)) {
			&error("cant-set-session");
			exit(0);
		}
	}
}


=item sanitise_go
Disallow redirecting outside current directory

0: url
r: url (possible broken)
=cut
sub sanitise_go($) {
	my $url = shift;
	if ($url =~ /http%3A/i || $url =~ /http:/i ||
			$url =~ /%2F/i || $url =~ /\//) {
		return "http://www.google.com/";
	} else {
		return $url;
	}
}


=item main
Main function
=cut
sub main() {
	# Updates cookie automagically
	my $sid = &Wanha::Session::get_id();
	my $phase = &Wanha::HTTP::get_numeric("set", 0);
	my $go = &Wanha::HTTP::get("go");
	$go = &sanitise_go($go) if (defined($go));

	if (defined($phase) || (! defined($sid) && ! defined($phase))) {
		$phase = 1 if (! defined($phase));
		&try_set_session($sid, $phase, $go);
	}

	# Cookie set

	if (defined($go)) {
		&Wanha::HTTP::redirect($go);
	}

	&Wanha::HTTP::print_header();
	&Wanha::HTML::print_header();

	$sid = "<em>undefined</em>" unless defined($sid);
	print "<tt>sid = $sid</tt><br />\n";

	&Wanha::HTML::print_footer();
}


&main();
exit(0);
